@@ -72,17 +72,17 @@ class MessageHistory(BaseModel):
 
 	system_message: BaseMessage | None = None
 	state_message: BaseMessage | None = None
-	consistent_messages: list[BaseMessage] = Field(default_factory=list)
+	context_messages: list[BaseMessage] = Field(default_factory=list)
 	model_config = ConfigDict(arbitrary_types_allowed=True)
 
 	def get_messages(self) -> list[BaseMessage]:
-		"""Get all messages"""
+		"""Get all messages in the correct order: system -> state -> contextual"""
 		messages = []
 		if self.system_message:
 			messages.append(self.system_message)
 		if self.state_message:
 			messages.append(self.state_message)
-		messages.extend(self.consistent_messages)
+		messages.extend(self.context_messages)
 
 		return messages
 
